@@ -16,18 +16,22 @@ from geneticengine.core.decorators import get_gengy
 from geneticengine.core.grammar import Grammar
 from geneticengine.core.random.sources import Source
 from geneticengine.core.representations.api import Representation
+from geneticengine.core.representations.tree.utils import GengyList
 from geneticengine.core.representations.tree.utils import relabel_nodes
 from geneticengine.core.representations.tree.utils import relabel_nodes_of_trees
 from geneticengine.core.tree import TreeNode
 from geneticengine.core.utils import build_finalizers
 from geneticengine.core.utils import get_arguments
 from geneticengine.core.utils import get_generic_parameter
+from geneticengine.core.utils import has_annotated_crossover
+from geneticengine.core.utils import has_annotated_mutation
 from geneticengine.core.utils import is_abstract
+from geneticengine.core.utils import is_annotated
 from geneticengine.core.utils import is_generic_list
+from geneticengine.core.utils import strip_annotations
 from geneticengine.exceptions import GeneticEngineError
 from geneticengine.metahandlers.base import is_metahandler
 
-
 T = TypeVar("T")
 
 
@@ -67,7 +71,6 @@ class SMTResolver:
     @staticmethod
     def add_clause(claus, recs: dict[str, Callable]):
         SMTResolver.clauses.extend(claus)
-        # print(recs)
         for k, v in recs.items():
             SMTResolver.receivers[k] = v
 
@@ -219,7 +222,6 @@ def PI_Grow(
         return valid_productions
 
     while prodqueue:
-        # print(nRecs[0])
         next_type, next_finalizer, depth, ident, ctx = r.pop_random(prodqueue)
         if next_type in g.recursive_prods:
             nRecs[0] -= 1
@@ -252,7 +254,14 @@ def random_node(
     return method(r, g, max_depth, starting_symbol)
 
 
-def mk_save_init(starting_symbol: Callable, receiver: Callable):
+def mk_save_init(starting_symbol: Any, receiver: Callable):
+    if isinstance(starting_symbol, type):
+        pass
+    elif isinstance(starting_symbol, GengyList):
+        starting_symbol = starting_symbol.new_like
+    else:
+        starting_symbol = type(starting_symbol)
+
     def fin_recv(*x):
         built = starting_symbol(*x)
         built.gengy_init_values = x
@@ -373,8 +382,6 @@ def expand_node(
 
                 def fn(val, name=name):
                     pass
-                    # print(f"{name}={val}")
-                    # SMTResolver.register_const(name, val)
 
                 l.append(fn)
 
@@ -413,6 +420,36 @@ def mutate_inner(
     if i.gengy_nodes > 0:
         c = r.randint(0, i.gengy_nodes - 1)
         if c == 0:
+            # If Metahandler mutation exists, the mutation process is different
+            args_with_specific_mutation = [
+                has_annotated_mutation(arg[1]) for arg in get_arguments(i)
+            ]
+            if any(args_with_specific_mutation):
+                mutation_possibilities = len(args_with_specific_mutation)
+                mutation_choice = r.randint(
+                    0,
+                    mutation_possibilities,
+                )  # including 0 so that the whole node can also be replaced
+                if mutation_choice == mutation_possibilities:
+                    pass  # Replace whole node
+                else:
+                    (index, arg_to_be_mutated) = [
+                        (kdx, arg[1])
+                        for kdx, arg in enumerate(get_arguments(i))
+                        if args_with_specific_mutation[kdx]
+                    ][mutation_choice]
+                    args = list(i.gengy_init_values)
+                    args[index] = arg_to_be_mutated.__metadata__[0].mutate(  # type: ignore
+                        r,
+                        g,
+                        random_node,
+                        max_depth - 1,
+                        ty,
+                        method=Grow,
+                        current_node=args[index],
+                    )
+                    return mk_save_init(type(i), lambda x: x)(*args)
+
             replacement = random_node(r, g, max_depth, ty, method=Grow)
             return replacement
         else:
@@ -420,7 +457,7 @@ def mutate_inner(
                 max_depth -= g.abstract_dist_to_t[ty][type(i)]
             max_depth -= 1
             args = list(i.gengy_init_values)
-            for idx, (field, field_type) in enumerate(get_arguments(i)):
+            for idx, (_, field_type) in enumerate(get_arguments(i)):
                 child = args[idx]
                 if hasattr(child, "gengy_nodes"):
                     count = child.gengy_nodes
@@ -435,7 +472,7 @@ def mutate_inner(
                         break
                     else:
                         c -= count
-            return mk_save_init(type(i), lambda x: x)(*args)
+            return mk_save_init(i, lambda x: x)(*args)
     else:
         return random_node(r, g, max_depth, ty, method=Grow)
 
@@ -466,16 +503,23 @@ def find_in_tree(g: Grammar, ty: type, o: TreeNode, max_depth: int):
 
                 return depth <= max_depth
 
-            if not isinstance(ty, type):
-                raise NotImplementedError("")
             if ty in t.__bases__:
                 vals = o.gengy_types_this_way[t]
                 if vals:
                     yield from filter(is_valid, vals)
 
 
-def get_height(n: TreeNode):
-    pass
+def find_in_tree_exact(g: Grammar, ty: type, o: TreeNode, max_depth: int):
+    if hasattr(o, "gengy_types_this_way"):
+
+        vals = o.gengy_types_this_way[ty]
+        if vals:
+
+            def is_valid(node):
+                _, depth, _ = relabel_nodes(node, g)
+                return depth <= max_depth
+
+            yield from filter(is_valid, vals)
 
 
 def tree_crossover_inner(
@@ -490,6 +534,41 @@ def tree_crossover_inner(
         c = r.randint(0, i.gengy_nodes - 1)
         if c == 0:
             replacement = None
+            args_with_specific_crossover = [
+                has_annotated_crossover(arg[1]) for arg in get_arguments(i)
+            ]
+            if any(args_with_specific_crossover):
+                crossover_possibilities = len(args_with_specific_crossover)
+                crossover_choice = r.randint(
+                    0,
+                    crossover_possibilities,
+                )  # including 0 so that the whole node can also be replaced
+                options = list(find_in_tree_exact(g, type(i), o, max_depth))
+                if crossover_choice == crossover_possibilities or (
+                    not options
+                ):  # make sure there are options!
+                    pass  # Replace whole node
+                else:
+                    (index, arg_to_be_crossovered) = [
+                        (kdx, arg)
+                        for kdx, arg in enumerate(get_arguments(i))
+                        if args_with_specific_crossover[kdx]
+                    ][crossover_choice]
+                    args = list(i.gengy_init_values)
+                    args[index] = (
+                        arg_to_be_crossovered[1]
+                        .__metadata__[0]  # type: ignore
+                        .crossover(
+                            r,
+                            g,
+                            options,
+                            arg_to_be_crossovered[0],
+                            ty,
+                            current_node=args[index],
+                        )
+                    )
+                    return mk_save_init(type(i), lambda x: x)(*args)
+
             options = list(find_in_tree(g, ty, o, max_depth))
             if options:
                 replacement = r.choice(options)
@@ -518,7 +597,7 @@ def tree_crossover_inner(
                         break
                     else:
                         c -= count
-            return mk_save_init(type(i), lambda x: x)(*args)
+            return mk_save_init(i, lambda x: x)(*args)
     else:
         return i
 
